#include <stdio.h>
#include <math.h>

#include <qlib2.h>

#include "params.h"
#include "st_info.h"
#include "externals.h"
#include "procs.h"

static int  duration_val = 0;
static char duration_unit = 'M';

/************************************************************************/
/*  parse_duration:							*/
/*	Parse duration string into external variables:			*/
/*	    duration_val:   numeric value for duration.			*/
/*	    duration_unit:  character denomination for duration.	*/
/*  return:								*/
/*	    0 on success, -1 on error.					*/
/************************************************************************/
int parse_duration
   (char	*str)		/* duration string.			*/
{
    char *p;
    int error = 0;
    int i;
    duration_val = strtol (str, &p, 10);
    duration_unit = p ? *p : 'M';
    /* Check on the integrity of the duration string.			*/
    switch (duration_unit) {
      case 'y':
      case 'Y':
	if (duration_val != 1) ++error;		/* only allow 1 year.	*/
	break;
      case 'm':
	i = 12 / duration_val;
	if (i * duration_val != 12) ++error;
	break;
      case 'd':
	i = duration_val;
	if (i <= 0 || i > 366) ++error;
      case 'H':
	i = 24 / duration_val;
	if (i * duration_val != 24) ++error;
	break;
      case 'M':
	i = 60 / duration_val;
	if (i * duration_val != 60) ++error;
	break;
      case 'S':
	i = 50 / duration_val;
	if (i * duration_val != 60) ++error;
	break;
      default:
	++error;
	break;
    }
    if (error || duration_val <= 0) {
	fprintf (info, "Error: invalid duration: %s\n", str);
	return (-1);
    }
    return (0);
}


/************************************************************************/
/*  compute_duration:							*/
/*	Compute start and end duration based on time in DATA_HDR.	*/
/*	Set external variables:						*/
/*	    beg_duration, end_duration.					*/
/************************************************************************/
void compute_duration 
   (DATA_HDR	*hdr)		/* DATA_HDR for record.			*/
{
    EXT_TIME bt, et;
    bt = int_to_ext(hdr->begtime);
    switch (duration_unit) {
      case 'y':
      case 'Y':
	bt.month = 1;
	bt.day = 1; 
	bt.hour = bt.minute = bt.second = bt.usec = 0;
	bt.doy = mdy_to_doy (bt.month, bt.day, bt.year);
	et = bt;
	et.year += duration_val;
	et = normalize_ext (et);
	break;
      case 'm':
	bt.month -= (bt.month-1) % duration_val;
	bt.day = 1; 
	bt.hour = bt.minute = bt.second = bt.usec = 0;
	bt.doy = mdy_to_doy (bt.month, bt.day, bt.year);
	et = bt;
	et.month += duration_val;
	et.doy = mdy_to_doy (et.month, et.day, et.year);
	et = normalize_ext (et);
	break;
      case 'd':
	bt.doy -= (bt.doy-1) % duration_val;
	bt.hour = bt.minute = bt.second = bt.usec = 0;
	dy_to_mdy (bt.doy, bt.year, &bt.month, &bt.day);
	et = bt;
	et.doy += duration_val;
	et = normalize_ext (et);
	break;
      case 'H':
	bt.hour -= bt.hour % duration_val;
	bt.minute = bt.second = bt.usec = 0;
	et = bt;
	et.hour += duration_val;
	et = normalize_ext (et);
	break;
      case 'M':
	bt.minute -= bt.minute % duration_val;
	bt.second = bt.usec = 0;
	et = bt;
	et.minute += duration_val;
	et = normalize_ext (et);
	break;
      case 'S':
	bt.second -= bt.second % duration_val;
	bt.usec = 0;
	et = bt;
	et.second += duration_val;
	et = normalize_ext (et);
	break;
      default:
	fprintf (info, "Error: invalid duration unit: %c\n", duration_unit);
	exit(1);
    }
    beg_duration = ext_to_int(bt);
    end_duration = ext_to_int(et);
    init_duration = 1;
}

/************************************************************************/
/*  after_duration:							*/
/*	Determine whether start time is after (incl) end duration.	*/
/*  return:								*/
/*	1 on true, 0 on false.						*/
/************************************************************************/
int after_duration
   (DATA_HDR	*hdr)		/* DATA_HDR for record.			*/
{
    if (tdiff(hdr->begtime,end_duration) >= 0) return (1);
    else return (0);
}

/************************************************************************/
/*  cross_duration:							*/
/*	Determine whether record crosses a duration boundary.		*/
/*  return:								*/
/*	1 on true, 0 on false.						*/
/************************************************************************/
int cross_duration
   (DATA_HDR	*hdr)		/* DATA_HDR for record.			*/
{
    if ((tdiff(beg_duration,hdr->begtime) > 0 && tdiff(beg_duration,hdr->endtime) <= 0) ||
	(tdiff(end_duration,hdr->begtime) > 0 && tdiff(end_duration,hdr->endtime) <= 0))
    return (1);
    else return (0);
}

/************************************************************************/
/*  intersect_duration:							*/
/*	Determine sample offset within record to split for duration.	*/
/*  return:								*/
/*	sample_offset in record (1 to nsamples-1)
/************************************************************************/
int intersect_duration
   (DATA_HDR	*hdr)		/* DATA_HDR for record.			*/
{
    double diff, d;
    int n;
    if ((diff=tdiff(beg_duration,hdr->begtime)) > 0 && tdiff(beg_duration,hdr->endtime) <= 0) ;
    else if ((diff=tdiff(end_duration,hdr->begtime)) > 0 && tdiff(end_duration,hdr->endtime) <= 0) ;
    else {
	fprintf (info, "Error: record does not intersect duration\n");
	exit (1);
    }
    d = dsamples_in_time2 (hdr->sample_rate, hdr->sample_rate_mult, diff);
    n = ceil(d);
    if (n <= 0 || n >= hdr->num_samples) {
	fprintf (info, "Error: computing intersect_duration: %d for record with %d samples\n",
		 n, hdr->num_samples);
	exit (1);
    }
    return (n);
}

/************************************************************************/
/*  build_filename:							*/
/*	Build a filename from the template format.			*/
/************************************************************************/
char *build_filename
   (DATA_HDR	*hdr,		/* DATA_HDR for record.			*/
    char	*pattern)	/* filename pattern.			*/
{
    static char name[2048];
    char *ip, *op;
    EXT_TIME et;
    ip = pattern;
    op = name;
    switch (duration_date_flag) {
      case 'n':
	et = int_to_ext(beg_duration); 
	break;
      case 'd':
	et = int_to_ext(hdr->begtime); 
	break;
      default:	
	fprintf (info, "Error: Invalid duration date flag: %c\n", 
			 duration_date_flag);
	exit(1);
    }

    while (*ip != '\0') {
	switch (*ip) {
	  case ('%'):
	    switch (*++ip) {
	    case ('S'):
		sprintf (op, "%s", hdr->station_id);
		uppercase(op);
		op += strlen(hdr->station_id); break;
	    case ('s'):
		sprintf (op, "%s", hdr->station_id);
		lowercase(op);
		op += strlen(hdr->station_id); break;
	    case ('N'):
		sprintf (op, "%s", hdr->network_id);
		uppercase(op);
		op += strlen(hdr->network_id); break;
	    case ('n'):
		sprintf (op, "%s", hdr->network_id);
		lowercase(op);
		op += strlen(hdr->network_id); break;
	    case ('L'):
		sprintf (op, "%s", hdr->location_id);
		uppercase(op);
		op += strlen(hdr->network_id); break;
	    case ('l'):
		sprintf (op, "%s", hdr->location_id);
		lowercase(op);
		op += strlen(hdr->network_id); break;
	    case ('C'):
		sprintf (op, "%s", hdr->channel_id);
		uppercase(op);
		op += strlen(hdr->channel_id); break;
	    case ('c'):
		sprintf (op, "%s", hdr->channel_id);
		lowercase(op);
		op += strlen(hdr->channel_id); break;
	    case ('Y'):
		sprintf (op, "%04d", et.year);
		op += 4; break;
	    case ('y'):
		sprintf (op, "%02d", et.year % 100);
		op += 2; break;
	    case ('m'):
		sprintf (op, "%02d", et.month);
		op += 2; break;
	    case ('d'):
		sprintf (op, "%02d", et.day);
		op += 2; break;
	    case ('j'):
		sprintf (op, "%03d", et.doy);
		op += 3; break;
	    case ('T'):
	    case ('t'):
		sprintf (op, "%02d%02d%02d", et.hour, 
			 et.minute, et.second);
		op += 6; break;
	    default:
		fprintf (info, "invalid format\n");
		sprintf (op, "%%%c", *ip);
		break;
	    }
	    ++ip; break;
	default:    *op++ = *ip++; break;
	}
    }
    *op = '\0';
    return(name);
}
